using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using ProcessManagerXNA.Processes;

namespace ProcessManagerXNA
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Process.ProcessManager processManager;

        Texture2D blankTexture;
        Color overlayColor;

        SoundEffect sound1;
        SoundEffect sound2;
        SoundEffect sound3;
        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";


            processManager = new Process.ProcessManager(this);
            this.Components.Add(processManager);
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();

            SoundProcess soundProcess1 = new SoundProcess(sound1);
            SoundProcess soundProcess2 = new SoundProcess(sound2);
            SoundProcess soundProcess3 = new SoundProcess(sound3);

            FadeScreenProcess fadeProcess = new FadeScreenProcess(this, Color.Red, TimeSpan.FromSeconds(5));

            soundProcess1.AttachChild(soundProcess2);
            soundProcess2.AttachChild(soundProcess3);
            soundProcess3.AttachChild(fadeProcess);

            processManager.AttachProcess(soundProcess1);
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            blankTexture = new Texture2D(GraphicsDevice, 1, 1);
            Color[] color = new Color []{Color.White};
            blankTexture.SetData(color);

            sound1 = Content.Load<SoundEffect>("sound1");
            sound2 = Content.Load<SoundEffect>("sound2");
            sound3 = Content.Load<SoundEffect>("sound3");
        }

        public void SetOverlayColor(Color color)
        {
            overlayColor = color;
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            
            // TODO: Add your drawing code here
            spriteBatch.Begin();
            
            if (overlayColor != Color.Transparent)
            {
                spriteBatch.Draw(blankTexture, GraphicsDevice.Viewport.Bounds, overlayColor);
            }

            spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}
